/*********************************************************************
    This file is part of Siworin. Copyright © 2021 by Ruud Harmsen.

    Siworin is a simple word indexer and local search engine. See
    https://rudhar.com/sfreview/siworin/#siw04 and
    https://rudhar.com/sfreview/siworin/toolsrc/siworin .

    Siworin is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
    by the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    Siworin is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Siworin.  If not, see <https://www.gnu.org/licenses/>.
  ********************************************************************/

/* Siworin - simple word indexer
   Written by Ruud Harmsen, July 2021

   siworin-pathoffs takes as input a file that contains pathnames,
   one per line, of HTML files (or other compatible material).
   The output is a text file that has equal-length lines, each
   containing a byte offset to a line in the other file.
   Each offset has exactly 8 hexadecimal digits (32 bits), (see
   SIWORIN_OFFSET_HEXDIGITS in siworin.h) so the maximum supported size
   of the file with pathnames is 4G bytes. (A practical example is my
   own site's, which has a path name list smaller than 100.000 bytes.)

   The purpose of this is, that a short index number can refer to
   the much longer path name, without having to store that longer
   name every time again. This is useful for the eventual list of
   words, which contains path-offset pairs of where that word occurs.
   That word list should not become to large, for a reasonable
   search performance.

   The default naming of the two files is siworin.paths and siworin.pathoffs
   in the current directory. However, if the first command line argument
   is given, that is used as a replacement for "siworin" in those names.
   E.g. when called as "p /var/www/cgi-bin/someindex", the two files will be
   /var/www/cgi-bin/someindex.paths and
   /var/www/cgi-bin/someindex.pathoffs .
 */

#include <stdio.h>
#include <string.h>
#include "siworin.h"
#include "siworin-files.h"
#include "siworin-config.h"

static void Usage (char *progname)
{
   fprintf(stderr, "Usage:\n");
   fprintf(stderr, "%s\n", progname);
   fprintf(stderr, "%s -cCONFIGFILE\n", progname);
   fprintf(stderr, "%s -c CONFIGFILE\n", progname);
   fprintf(stderr, "%s --config CONFIGFILE\n", progname);
}

int main (int argc, char **argv)
{
   static char path[SIWORIN_MAXPATHLEN];
   long curpos;

   if (argc > 3)
   {
      Usage(argv[0]);
      return 1;
   }
   siworin_args_to_config_path(argv);
   if (siworin_fileopen_paths(
       siworin_config_value(SIWORIN_CONFIG_INDEX_FILES_PREFIX),
       "r", "w"))
      SIWORIN_FAIL("fopens");

   curpos = ftell(siworin_fp_paths);
   while (fgets(path, sizeof path, siworin_fp_paths) != NULL)
   {
      fprintf(siworin_fp_pathoffs, "%0*lx\n", (int)SIWORIN_OFFSET_HEXDIGITS, curpos);
      curpos = ftell(siworin_fp_paths);
   }

   siworin_fileopen_close();
   siworin_config_freeall();

   return 0;
}
